import os
import weakref
import importlib.util

class Cached(type):
    '''
    Class used as a metaclass to generate cached instances.
    Make sure create single instance with the same arguments.
    '''
    def __init__(self, *args, **kwargs):
        super(Cached, self).__init__(*args, **kwargs)
        self.__cache = weakref.WeakValueDictionary()

    def __call__(self, *args):
        if args in self.__cache:
            return self.__cache[args]
        else:
            obj = super(Cached, self).__call__(*args)
            self.__cache[args] = obj
            return obj


def load_module(module_name, path):
    spec = importlib.util.spec_from_file_location(module_name, os.path.join(path, module_name + '.py'))
    module = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(module)
    return module



import getpass
import json
import logging

import os
import stat

from eth_utils import is_hex, decode_hex, encode_hex, remove_0x_prefix
import eth_keyfile

logger = logging.getLogger('dbot.' + os.path.splitext(os.path.basename(__file__))[0])


def check_permission_safety(path):
    """Check if the file at the given path is safe to use as a state file.

    This checks that group and others have no permissions on the file and that the current user is
    the owner.
    """
    f_stats = os.stat(path)
    return (f_stats.st_mode & (stat.S_IRWXG | stat.S_IRWXO)) == 0 and f_stats.st_uid == os.getuid()


def get_private_key(pk_file, pw_file=None):
    """Open a JSON-encoded private key and return it

    If a password file is provided, uses it to decrypt the key. If not, the
    password is asked interactively. Raw hex-encoded private keys are supported,
    but deprecated."""

    is_hex_key = is_hex(pk_file) and len(remove_0x_prefix(pk_file)) == 64
    is_path = os.path.exists(pk_file)
    assert is_hex_key or is_path, 'Private key must either be a hex key or a file path.'

    # Load private key from file if none is specified on command line.
    if is_path:
        private_key = load_pk(pk_file, pw_file)
        assert private_key is not None, 'Could not load private key from file.'
        return private_key
    else:
        # TODO make sure '0x'
        return pk_file


def load_pk(pk_file, pw_file=None):
    assert pk_file, pk_file
    if not os.path.exists(pk_file):
        logger.fatal("%s: no such file", pk_file)
        return None
    #  if not check_permission_safety(pk_file):
    #      logger.fatal("Private key file %s must be readable only by its owner.", pk_file)
    #      return None
    #
    #  if pw_file and not check_permission_safety(pw_file):
    #      logger.fatal("Password file %s must be readable only by its owner.", pw_file)
    #      return None

    with open(pk_file) as keyfile:
        private_key = keyfile.readline().strip()

        if is_hex(private_key) and len(decode_hex(private_key)) == 32:
            logger.warning("Private key in raw format. Consider switching to JSON-encoded")
        else:
            if pw_file:
                with open(pw_file) as password_file:
                    password = password_file.readline().strip()
            else:
                password = getpass.getpass("Enter the private key password: ")
            private_key = eth_keyfile.extract_key_from_keyfile(pk_file, password).hex()
    return private_key



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
paste swagger specification file
"""
import os
import json

from .tool import Cached

class SwaggerParser(metaclass=Cached):
    def __init__(self, filename, filehander=None):
        if filehander is None:
            with open(filename, 'r') as fh:
                self._spec = json.load(fh)
        else:
            self._spec = json.load(filehander)
        self._load_spec()

    def _load_spec(self):
        # TODO use schema to check if specification is valid.
        if self._spec.get('swagger') != '2.0':
            raise Exception('Dbot specification only support swagger 2.0 now.')
        try:
            self.api_host = self._spec['host']
            self.protocol = 'https' if 'https' in self._spec['schemes'] else 'http'
            self.endpoints = []
            basePath = self._spec.get('basePath', '')
            for path, path_value in self._spec['paths'].items():
                uri = basePath + path
                for method, method_value in path_value.items():
                    self.endpoints.append({
                        'uri': uri,
                        'method': method
                    })
        except Exception as err:
            raise Exception('Parse dbot specification failed: {}'.format(err))

    def dumps(self):
        return json.dumps(self._spec)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="dbot-manager",
    version="0.0.3",
    author="ovsoil",
    author_email="huaxin.yu@atmatrix.org",
    description="Develop tools for ATN DBot",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ATNIO/dbot-manager",
    packages=setuptools.find_packages(),
    install_requires=[
        'click==6.7',
        'eth_utils==1.2.2',
        'eth_keyfile==0.5.1',
        'typing==3.6.6',
        'eth_account==0.3.0',
        'web3==4.7.2',
        'requests==2.19.1'
    ],
    package_data={
        '': ['contracts.json']
    },
    scripts=[
        'dbot-service'
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



from .tool import (
    Cached,
    load_module
)

from .account import (
    check_permission_safety,
    get_private_key
)

from .swagger import (
    SwaggerParser
)


__all__ = [
    Cached,
    load_module,
    check_permission_safety,
    get_private_key,
    SwaggerParser
]
